(* MultiCopy
   =========
   Copy a file to all subdirectories of a root where the same file already exist

    Dr. J. Rathlev, D-24222 Schwentinental (info(a)rathlev-home.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   J. Rathlev, Jan. 2008
   last changes: May 2022
   *)

unit MuCoMain;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes,
  Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, Vcl.Buttons,
  HListBox, Vcl.ComCtrls;

const
  ProgName = 'Multi file copy';
  Vers = '3.0';
  CopRgt = ' 2008-2022 Dr. J. Rathlev, 24222 Schwentinental';
  EmailAdr = 'kontakt(a)rathlev-home.de';


type
  THauptForm = class(TForm)
    SourceDir: THistoryCombo;
    FileBtn: TSpeedButton;
    DestDirBtn: TSpeedButton;
    DestDir: THistoryCombo;
    Label1: TLabel;
    EndeBtn: TBitBtn;
    LogBox: TListBox;
    FolderIdEdit: THistoryCombo;
    Label4: TLabel;
    Label5: TLabel;
    Label3: TLabel;
    SourceFiles: THistoryCombo;
    sbSourceDir: TSpeedButton;
    lvFiles: TListView;
    Label2: TLabel;
    btnCopy: TBitBtn;
    btInfo: TBitBtn;
    cbOverwrite: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure EndeBtnClick(Sender: TObject);
    procedure InfoBtnClick(Sender: TObject);
    procedure FileBtnClick(Sender: TObject);
    procedure DestDirBtnClick(Sender: TObject);
    procedure btnCopyClick(Sender: TObject);
    procedure sbSourceDirClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private-Deklarationen }
    ProgVersName,
    ProgVersDate,
    ProgVers,
    AppPath,UserPath,
    IniName          : string;
    function GetSourceDir : boolean;
  public
    { Public-Deklarationen }
  end;

var
  HauptForm: THauptForm;

implementation

{$R *.dfm}

uses WinUtils, ShellDirDlg, System.IniFiles, Winapi.ShlObj, System.StrUtils,
  WinApiUtils, FileUtils, StringUtils, GnuGetText, InitProg, LangUtils, SelectFilesDlg;

const
  IniExt = 'ini';

  (* INI-Sektionen *)
  CfGSekt = 'Config';
  SourceSekt = 'Source';
  DestSekt = 'Dest';
  FileSekt = 'Files';
  IdSekt = 'SubFolders';
  iniOvrWrt = 'Overwrite';

  (* INI-Variablen *)

{ ------------------------------------------------------------------- }
(* INI-File lesen *)
procedure THauptForm.FormCreate(Sender: TObject);
begin
  TranslateComponent(self);
  InitPaths(AppPath,UserPath);
  InitVersion(ProgName,Vers,CopRgt,3,3,ProgVersName,ProgVers,ProgVersDate);
  IniName:=Erweiter(AppPath,PrgName,IniExt);
  with TIniFile.Create(IniName) do begin
    cbOverwrite.Checked:=ReadBool(CfgSekt,iniOvrWrt,true);
    Free;
    end;
  SourceDir.LoadFromIni (IniName,SourceSekt);
  SourceDir.ItemIndex:=0;
  DestDir.LoadFromIni (IniName,DestSekt);
  DestDir.ItemIndex:=0;
  SourceFiles.LoadFromIni (IniName,FileSekt);
  SourceFiles.ItemIndex:=0;
  FolderIdEdit.LoadFromIni(IniName,IdSekt);
  FolderIdEdit.ItemIndex:=0;
  Caption:=_('Multiple copies of files')+ProgVers;
  end;

procedure THauptForm.FormDestroy(Sender: TObject);
begin
  with TIniFile.Create(IniName)do begin
    WriteBool(CfgSekt,iniOvrWrt,cbOverwrite.Checked);
    Free;
    end;
  end;

{ ------------------------------------------------------------------- }
(* Programmende *)
procedure THauptForm.EndeBtnClick(Sender: TObject);
begin
  Close;
  end;

procedure THauptForm.InfoBtnClick(Sender: TObject);
begin
  InfoDialog (ProgName,_('Multiple copies of files')+ProgVers+' - '
           +ProgVersDate+sLineBreak+CopRgt
           +sLineBreak+'E-Mail: '+EmailAdr);
  end;

{ ------------------------------------------------------------------- }
function THauptForm.GetSourceDir : boolean;
var
  s : string;
begin
  s:=GetExistingParentPath(SourceDir.Text,UserPath);
  if ShellDirDialog.Execute (_('Folder to source files'),true,true,true,'',s) then begin
    with SourceDir do begin
      Text:=s; AddItem(s);
      end;
    Result:=true;
    end
  else Result:=false;
  end;

procedure THauptForm.sbSourceDirClick(Sender: TObject);
begin
  GetSourceDir;
  end;

procedure THauptForm.FileBtnClick(Sender: TObject);
var
  t : string;
begin
  if ((length(SourceDir.Text)>0) and DirectoryExists(SourceDir.Text))
        or GetSourceDir then begin
    if SelectFiles(_('Select source files'),SourceDir.Text,t) then
      with SourceFiles do begin
        Text:=t; AddItem(Text);
//    InitialDir:=SourceDir.Text;
//    Filename:='';
//    Filter:=_('All files')+'|*.*';
//    Title:=_('Select source files');
//    if Execute then begin
//      for i:=0 to Files.Count-1 do begin
//        t:=MakeQuotedStr(MakeRelativePath(SourceDir.Text,Files[i]),[Space]);
//        with SourceFiles do if (i=0) then Text:=t else Text:=Text+Comma+t;
//        end;
      end;
    end;
  end;

procedure THauptForm.DestDirBtnClick(Sender: TObject);
var
  s : string;
begin
  s:=GetExistingParentPath(DestDir.Text,UserPath);
  if ShellDirDialog.Execute (_('Select root directory'),false,true,false,'',s) then begin
    with DestDir do begin
      Text:=s; AddItem(s);
      end;
    end;
  end;

{ ------------------------------------------------------------------- }
procedure THauptForm.btnCopyClick(Sender: TObject);
var
  fcnt,ecnt : integer;
  s,sf      : string;

  procedure CopyToDir (const SDir,Filename,FolderId,DDir : string);
  var
    DirInfo    : TSearchRec;
    Findresult : integer;
    sd,ss      : string;
  begin
    FindResult:=FindFirst (Erweiter(DDir,'*','*'),faAnyFile,DirInfo);
    while FindResult=0 do with DirInfo do begin
      if NotSpecialDir(Name) then begin
        if((Attr and faDirectory)<>0) then CopyToDir(SDir,Filename,FolderId,Erweiter(DDir,Name,''));
        end;
      FindResult:=FindNext (DirInfo);
      end;
    FindClose(DirInfo);
    if AnsiContainsText(DDir,IncludeTrailingPathDelimiter(FolderId)) or AnsiEndsText(FolderId,DDir) then begin
      sd:=AddPath(DDir,Filename); ss:=AddPath(SDir,Filename);
      if FileExists(sd) then begin
        if not cbOverwrite.Checked or (FileTimeToUnixTime(GetFileLastWriteTime(ss))-
                     FileTimeToUnixTime(GetFileLastWriteTime(sd))>2) then begin
          try
            CopyFileTS(ss,sd);
            LogBox.Items.Add(sd);
            inc(fcnt);
          except
            on E:EInOutError do begin
              ErrorDialog('',E.Message);
              LogBox.Items.Add(_('==> Error: ')+sd);
              inc(ecnt);
              end;
            end;
          end
        else LogBox.Items.Add(_('Skipped: ')+sd);
        end;
      end;
    end;

begin
  with SourceDir do AddItem(Text);
  with SourceFiles do AddItem(Text);
  with DestDir do AddItem(Text);
  with FolderIdEdit do AddItem(Text);
  LogBox.Clear; lvFiles.Clear;
  s:=SourceFiles.Text;
  repeat
    fcnt:=0; ecnt:=0;
    sf:=ReadNxtQuotedStr(s,Comma,Quote);
    if FileExists(AddPath(SourceDir.Text,sf)) then begin
      CopyToDir(SourceDir.Text,sf,PathDelim+FolderIdEdit.Text,DestDir.Text);
      with lvFiles.Items.Add do begin
        Caption:=sf;
        SubItems.Add(IntToStr(fcnt));
        SubItems.Add(IntToStr(ecnt));
        end;
      end
    else begin
      LogBox.Items.Add(Format(_('==> Error: ')+_('not found')+Colon+'%s',[sf]));
      with lvFiles.Items.Add do begin
        Caption:=sf;
        SubItems.Add(_('not found'));
        end;
      end;
    until length(s)=0;
  end;

end.
